function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var program = _interopDefault(require('commander'));
var ErrorStackParser = _interopDefault(require('error-stack-parser'));
var fetch = _interopDefault(require('cross-fetch'));
var logger = _interopDefault(require('npmlog'));
var envCi = _interopDefault(require('env-ci'));

// A type of promise-like that resolves synchronously and supports only one observer
var _Pact = function () {
  function _Pact() {}

  _Pact.prototype.then = function (onFulfilled, onRejected) {
    var result = new _Pact();
    var state = this.s;

    if (state) {
      var callback = state & 1 ? onFulfilled : onRejected;

      if (callback) {
        try {
          _settle(result, 1, callback(this.v));
        } catch (e) {
          _settle(result, 2, e);
        }

        return result;
      } else {
        return this;
      }
    }

    this.o = function (_this) {
      try {
        var value = _this.v;

        if (_this.s & 1) {
          _settle(result, 1, onFulfilled ? onFulfilled(value) : value);
        } else if (onRejected) {
          _settle(result, 1, onRejected(value));
        } else {
          _settle(result, 2, value);
        }
      } catch (e) {
        _settle(result, 2, e);
      }
    };

    return result;
  };

  return _Pact;
}(); // Settles a pact synchronously

function _settle(pact, state, value) {
  if (!pact.s) {
    if (value instanceof _Pact) {
      if (value.s) {
        if (state & 1) {
          state = value.s;
        }

        value = value.v;
      } else {
        value.o = _settle.bind(null, pact, state);
        return;
      }
    }

    if (value && value.then) {
      value.then(_settle.bind(null, pact, state), _settle.bind(null, pact, 2));
      return;
    }

    pact.s = state;
    pact.v = value;
    var observer = pact.o;

    if (observer) {
      observer(pact);
    }
  }
}

function _catch(body, recover) {
  try {
    var result = body();
  } catch (e) {
    return recover(e);
  }

  if (result && result.then) {
    return result.then(void 0, recover);
  }

  return result;
} // Asynchronously await a promise and pass the result to a finally continuation

var _earlyReturn = {}; // Asynchronously call a function and send errors to recovery continuation, skipping early returns

var _AsyncGenerator = function () {
  function _AsyncGenerator(entry) {
    this._entry = entry;
    this._pact = null;
    this._resolve = null;
    this._return = null;
    this._promise = null;
  }

  function _wrapReturnedValue(value) {
    return {
      value: value,
      done: true
    };
  }

  function _wrapYieldedValue(value) {
    return {
      value: value,
      done: false
    };
  }

  _AsyncGenerator.prototype[Symbol.asyncIterator || (Symbol.asyncIterator = Symbol("Symbol.asyncIterator"))] = function () {
    return this;
  };

  _AsyncGenerator.prototype._yield = function (value) {
    // Yield the value to the pending next call
    this._resolve(value && value.then ? value.then(_wrapYieldedValue) : _wrapYieldedValue(value)); // Return a pact for an upcoming next/return/throw call


    return this._pact = new _Pact();
  };

  _AsyncGenerator.prototype.next = function (value) {
    // Advance the generator, starting it if it has yet to be started
    var _this = this;

    return _this._promise = new Promise(function (resolve) {
      var _pact = _this._pact;

      if (_pact === null) {
        var _entry = _this._entry;

        if (_entry === null) {
          // Generator is started, but not awaiting a yield expression
          // Abandon the next call!
          return resolve(_this._promise);
        } // Start the generator


        _this._entry = null;
        _this._resolve = resolve;

        function returnValue(value) {
          _this._resolve(value && value.then ? value.then(_wrapReturnedValue) : _wrapReturnedValue(value));

          _this._pact = null;
          _this._resolve = null;
        }

        _entry(_this).then(returnValue, function (error) {
          if (error === _earlyReturn) {
            returnValue(_this._return);
          } else {
            var pact = new _Pact();

            _this._resolve(pact);

            _this._pact = null;
            _this._resolve = null;

            _resolve(pact, 2, error);
          }
        });
      } else {
        // Generator is started and a yield expression is pending, settle it
        _this._pact = null;
        _this._resolve = resolve;

        _settle(_pact, 1, value);
      }
    });
  };

  _AsyncGenerator.prototype.return = function (value) {
    // Early return from the generator if started, otherwise abandons the generator
    var _this = this;

    return _this._promise = new Promise(function (resolve) {
      var _pact = _this._pact;

      if (_pact === null) {
        if (_this._entry === null) {
          // Generator is started, but not awaiting a yield expression
          // Abandon the return call!
          return resolve(_this._promise);
        } // Generator is not started, abandon it and return the specified value


        _this._entry = null;
        return resolve(value && value.then ? value.then(_wrapReturnedValue) : _wrapReturnedValue(value));
      } // Settle the yield expression with a rejected "early return" value


      _this._return = value;
      _this._resolve = resolve;
      _this._pact = null;

      _settle(_pact, 2, _earlyReturn);
    });
  };

  _AsyncGenerator.prototype.throw = function (error) {
    // Inject an exception into the pending yield expression
    var _this = this;

    return _this._promise = new Promise(function (resolve, reject) {
      var _pact = _this._pact;

      if (_pact === null) {
        if (_this._entry === null) {
          // Generator is started, but not awaiting a yield expression
          // Abandon the throw call!
          return resolve(_this._promise);
        } // Generator is not started, abandon it and return a rejected Promise containing the error


        _this._entry = null;
        return reject(error);
      } // Settle the yield expression with the value as a rejection


      _this._resolve = resolve;
      _this._pact = null;

      _settle(_pact, 2, error);
    });
  };

  return _AsyncGenerator;
}();

var AnnotationLevel;

(function (AnnotationLevel) {
  AnnotationLevel["Notice"] = "notice";
  AnnotationLevel["Warning"] = "warning";
  AnnotationLevel["Failure"] = "failure";
})(AnnotationLevel || (AnnotationLevel = {}));

var detect = function () {
  var ref = envCi();
  var service = ref.service;
  var commit = ref.commit;
  var slug = ref.slug;
  var isCi = ref.isCi;

  if (!isCi) {
    throw new Error('CI environment not detected');
  }

  var ref$1 = service === 'codebuild' && process.env.CODEBUILD_SOURCE_REPO_URL ? process.env.CODEBUILD_SOURCE_REPO_URL.replace('https://github.com/', '').split('/') : slug.split('/');
  var owner = ref$1[0];
  var repo = ref$1[1];
  return {
    commit: commit,
    owner: owner,
    repo: repo
  };
};

var endpoint = process.env.CHECKS_PUBLISHER_URL || 'http://localhost:3000/publish';

var getOutput = function (target, data) {
  try {
    return Promise.resolve(_catch(function () {
      var formatter = require(("@monstrs/github-checks-formatter-" + target));

      return Promise.resolve(formatter.format(data));
    }, function (error) {
      var ref = ErrorStackParser.parse(error);
      var stackFrame = ref[0];
      return {
        title: error.name,
        summary: error.message,
        annotations: [{
          path: stackFrame.fileName || __filename,
          start_line: stackFrame.lineNumber || 0,
          start_column: stackFrame.columnNumber || 0,
          end_line: stackFrame.lineNumber || 0,
          end_column: stackFrame.columnNumber || 0,
          annotation_level: AnnotationLevel.Failure,
          raw_details: error.message,
          title: error.name,
          message: error.message
        }]
      };
    }));
  } catch (e) {
    return Promise.reject(e);
  }
};

var run = function (options, data) {
  try {
    logger.verbose(("Format with " + (options.target) + ", data:"), data);
    var ref = detect();
    var owner = ref.owner;
    var repo = ref.repo;
    var commit = ref.commit;
    return Promise.resolve(getOutput(options.target, data)).then(function (output) {
      var payload = {
        owner: owner,
        repo: repo,
        commit: commit,
        output: output,
        name: options.target
      };
      logger.verbose('Check payload:', payload);
      return Promise.resolve(fetch(endpoint, {
        method: 'POST',
        headers: {
          'Content-Type': 'application/json'
        },
        body: JSON.stringify(payload)
      })).then(function (response) {
        return response.json();
      });
    });
  } catch (e) {
    return Promise.reject(e);
  }
};

logger.heading = 'github-checks';
var stdin = process.openStdin();
var data = '';
stdin.on('data', function (chunk) { return data += chunk; });
stdin.on('end', function () {
  program.option('-t, --target [target]', 'Check target').option('-v, --verbose', 'Verbose output').parse(process.argv);

  if (program.verbose) {
    logger.level = 'verbose';
  }

  run({
    target: program.target
  }, data).then(function (response) { return logger.info(response.message); }).catch(function (error) {
    logger.error(error.message);
    process.exit(1);
  });
});
